﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Data.OleDb;
using System.IO;
using System.Drawing.Imaging;

// Add the database and the cover images to the project
// and set their "Copy to Output Directory" properties
// to "Copy if Newer."

// IMPORTANT: Do not open the database in Access or it
// may erase all of the image data.

namespace howto_save_image_in_database
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The database connection.
        private OleDbConnection Conn;

        // Display a list of titles.
        private void Form1_Load(object sender, EventArgs e)
        {
            // Compose the database file name.
            // This assumes it's in the executable's directory.
            string db_name = Application.StartupPath + "\\Books.mdb";

            // Connect to the database
            Conn = new OleDbConnection(
                "Provider=Microsoft.ACE.OLEDB.12.0;" +
                "Data Source=" + db_name + ";" +
                "Mode=Share Deny None");

            // Get the titles.
            OleDbCommand cmd = new OleDbCommand(
                "SELECT Title FROM Books ORDER BY Title",
                Conn);
            Conn.Open();
            OleDbDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                lstTitles.Items.Add(reader["Title"].ToString());
            }
            reader.Close();
            Conn.Close();
        }

        // Display information about the selected title.
        private void lstTitles_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstTitles.SelectedIndex < 0) return;
            btnSetImage.Enabled = true;

            // Make a command object to get information about the title.
            string title = lstTitles.SelectedItem.ToString().Replace("'", "''");
            OleDbCommand cmd = new OleDbCommand(
                "SELECT * FROM Books WHERE Title='" +
                title + "'",
                Conn);

            // Execute the command.
            cmd.Connection = Conn;
            Conn.Open();
            OleDbDataReader reader = cmd.ExecuteReader();
            reader.Read();

            // Display the text data.
            txtURL.Text = reader["URL"].ToString();
            txtYear.Text = reader["Year"].ToString();
            txtISBN.Text = reader["ISBN"].ToString();
            txtPages.Text = reader["Pages"].ToString();

            // Display the cover image.
            if (reader["CoverImage"] is DBNull)
            {
                picCover.Image = null;
            }
            else
            {
                Bitmap bm = BytesToImage((byte[])reader["CoverImage"]);
                picCover.Image = bm;
            }

            // Clean up.
            reader.Close();
            Conn.Close();
        }

        // Set the image for this record.
        private void btnSetImage_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Display the image.
                    Bitmap bm = new Bitmap(ofdImage.FileName);
                    picCover.Image = bm;

                    // Set the image in the database.
                    // The CoverImage field has type OLE Object.
                    string title = lstTitles.SelectedItem.ToString().Replace("'", "''");
                    OleDbCommand cmd = new OleDbCommand(
                        "UPDATE Books SET CoverImage=@Image WHERE Title='" +
                        title + "'",
                        Conn);

                    // Create a byte array holding the image.
                    byte[] image_bytes = ImageToBytes(bm, ImageFormat.Png);

                    // Add the image as a parameter.
                    OleDbParameter param = new OleDbParameter();
                    param.OleDbType = OleDbType.Binary;
                    param.ParameterName = "Image";
                    param.Value = image_bytes;
                    cmd.Parameters.Add(param);

                    // Execute the command (with no return value).
                    cmd.Connection = Conn;
                    Conn.Open();
                    cmd.ExecuteScalar();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                finally
                {
                    if (Conn.State != ConnectionState.Closed) Conn.Close();
                }
            }
        }

        // Convert an image into a byte array.
        private byte[] ImageToBytes(Image image, ImageFormat format)
        {
            using (MemoryStream image_stream = new MemoryStream())
            {
                image.Save(image_stream, format);
                return image_stream.ToArray();
            }
        }

        // Convert a byte array into an image.
        private Bitmap BytesToImage(byte[] bytes)
        {
            using (MemoryStream image_stream = new MemoryStream(bytes))
            {
                Bitmap bm = new Bitmap(image_stream);
                image_stream.Close();
                return bm;
            }
        }

        // Load images for all of the records.
        private void btnLoadAll_Click(object sender, EventArgs e)
        {
            string[] titles =
            {
                "Advanced Visual Basic Techniques",
                "Beginning Database Design Solutions",
                "Bug Proofing Visual Basic",
                "Custom Controls Library",
                "Microsoft Office Programming: A Guide for Experienced Developers",
                "Prototyping with Visual Basic",
                "Ready-to-Run Delphi Algorithms",
                "Ready-to-Run Visual Basic Algorithms, Second Edition",
                "Ready-to-Run Visual Basic Code Library",
                "Stephens' C# Programming with Visual Studio 2010 24-Hour Trainer",
                "Stephens' Visual Basic Programming 24-Hour Trainer",
                "Visual Basic .NET and XML",
                "Visual Basic .NET Database Programming",
                "Visual Basic 2005 Programmer's Reference",
                "Visual Basic 2010 Programmer's Reference",
                "Visual Basic Graphics Programming, Second Edition",
                "WPF Programmer's Reference",
            };
            string[] filenames =
            {
                @"Images\avbts.jpg",
                @"Images\db_design_s.jpg",
                @"Images\errs.jpg",
                @"Images\ccls.jpg",
                @"Images\offices.jpg",
                @"Images\protos.jpg",
                @"Images\das.jpg",
                @"Images\vbas.jpg",
                @"Images\vbcls.jpg",
                @"Images\24hour_t.jpg",
                @"Images\24hourvbs.jpg",
                @"Images\xmls.jpg",
                @"Images\vbdbs.jpg",
                @"Images\vb_prog_ref_1e_s.jpg",
                @"Images\vb_prog_ref_4e_s.png",
                @"Images\vbgp_1es.jpg",
                @"Images\wpf_prog_ref_t.png",
            };

            this.Cursor = Cursors.WaitCursor;

            string image_dir = Path.GetFullPath(
                Path.Combine(Application.StartupPath, "..\\..")) + "\\";

            // Load the images.
            for (int i = 0; i < titles.Length; i++)
            {
                LoadRecordImage(titles[i], image_dir + filenames[i]);
            }

            this.Cursor = Cursors.Default;
            MessageBox.Show("Loaded " + titles.Length + " images.");
        }

        // Load one record's picture.
        private void LoadRecordImage(string title, string filename)
        {
            try
            {
                // Get the image.
                Bitmap bm = new Bitmap(filename);

                // Set the image in the database.
                title = title.Replace("'", "''");
                OleDbCommand cmd = new OleDbCommand(
                    "UPDATE Books SET CoverImage=@Image WHERE Title='" +
                    title + "'",
                    Conn);

                // Create a byte array holding the image.
                byte[] image_bytes = ImageToBytes(bm, ImageFormat.Png);

                // Add the image as a parameter.
                OleDbParameter param = new OleDbParameter();
                param.OleDbType = OleDbType.Binary;
                param.ParameterName = "Image";
                param.Value = image_bytes;
                cmd.Parameters.Add(param);

                // Execute the command (with no return value).
                cmd.Connection = Conn;
                Conn.Open();
                cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Conn.Close();
            }
        }
    }
}
